# Plots and computations for 'futreviewSE' study.
# To be executed from top directory.
# Uses 'df' as global data frame variable throughout, plus various
# topic-specific ones.

# TODO:
# - simplify legend in the likert plots to say -5 (for 1) and 5 (for 10) only
# - see also top-level README.md


#-------------------- options and packages:

options(scipen=999)
options(repos=c("http://ftp5.gwdg.de/pub/misc/cran/"))

require(scales)

if(!require(agsemisc)){
  install.packages("agsemisc", dependencies = T)
  require(agsemisc)
}

if(!require(ggplot2)){
  install.packages("ggplot2", dependencies = T)
  require(ggplot2)
}

if(!require(devtools)){
  install.packages("devtools", dependencies = T)
  require(devtools)
}

if(!require(likert)){
  install_github('jbryer/likert')
  require(likert)
}


if(!require(psych)){
  install.packages("psych", dependencies = T)
  require(psych)
}

if(!require(reshape2)){
  install.packages("reshape2", dependencies = T)
  require(reshape2)
}


#-------------------- read and prepare data:

factorize.name = function(names) {
    paste(names, "f", sep="")
}

factorize10 = function(df, items) {
    newitems = factorize.name(items)
    for (i in seq(length(items))) {
        df[,newitems[i]] <- factor(df[,items[i]], levels = c(seq(1:10)))
    }
    df
}

rescale10 = function(a) {
    # map agreement 'a' from 1...10 scale onto -5...5 scale
    (a-1)/9*10 - 5
}

#--- main dataframe:
df <- read.csv(file.path("data", "02_survey-quantitative-export.csv"),
               na.strings=c("", "NA", "null", "NULL", "-", "N/A",
                            "-66", "-77", "-99"))

#--- "purpose of review":
purpose.items <- c("v_10", "v_11", "v_12", "v_13", "v_14", "v_15", 
                   "v_16", "v_17", "v_110")
df = factorize10(df, purpose.items)
purpose.likert <- data.frame(df[factorize.name(purpose.items)])
purpose.likert.describe <- describe(purpose.likert)
names(purpose.likert) <- c(
  v_10="(v_10)..make sure the research article is in tune with the scope of the venue.",
  v_11="(v_11)..make sure the research article is well-organized and easily understandable.",
  v_12="(v_12)..make sure the research results are valid.",
  v_13="(v_13)..make sure threats to the validity of research (and/or limitations) are properly addressed.",
  v_14="(v_14)..make sure the title, abstract, discussion of results, and conclusions in an article do not mislead or exaggerate.",
  v_15="(v_15)..protect the reputation of the publication venue.",
  v_16="(v_16)..detect plagiarism and self-plagiarism.",
  v_17="(v_17)..make sure the research has sufficient relevance.",
  v_110="(v_110)..help authors to become better researchers.")

#--- "current quality of reviews":
QPgood = na.omit(df$v_32)
QPreasonable = na.omit(df$v_33)
QPunhelpful = na.omit(df$v_34)
QPfaulty = na.omit(df$v_35)
quality = c(rep("good", length(QPgood)),
            rep("reasonable", length(QPreasonable)),
            rep("unhelpful", length(QPunhelpful)),
            rep("grossly faulty", length(QPfaulty)))
percent = c(QPgood, QPreasonable, QPunhelpful, QPfaulty)
reviewstatusquo.df = data.frame(quality=quality, percent=percent)
reviewstatusquo.df$quality = factor(reviewstatusquo.df$quality,
    levels=rev(c("good", "reasonable", "unhelpful", "grossly faulty")))

#--- "how open should reviewing be" (blinding):
df$AknowR = rescale10(df$v_104)
df$RknowA = -rescale10(df$v_105)
df$RknowR = -rescale10(df$v_106)
df$opennessChg = (df$AknowR + -df$RknowA + -df$RknowR) / 3
df$openness = (df$AknowR + df$RknowA + df$RknowR) / 3
reviewopenness.items <- c("v_104", "v_105", "v_106")
df = factorize10(df, reviewopenness.items)
reviewopenness.likert <- data.frame(df[factorize.name(reviewopenness.items)])
reviewopenness.likert.describe <- describe(reviewopenness.likert)
names(reviewopenness.likert) <- c(
  v_104="(v_104) Authors should know who the reviewers of an article are (signed).",
  v_105="(v_105) Reviewers should not reliably know who the authors of an article are.",
  v_106="(v_106) Reviewers should not reliably know who their co-reviewers are."
)

#--- "how public should reviewing be":
df$publicness = rescale10((df$v_107 + df$v_108 + df$v_109) / 3)
df$pubreviews = rescale10(df$v_107)
publicationopenness.items <- c("v_107", "v_108", "v_109")
df = factorize10(df, publicationopenness.items)
publicationopenness.likert <- data.frame(df[factorize.name(publicationopenness.items)])
publicationopenness.likert.describe <- describe(publicationopenness.likert)
names(publicationopenness.likert) <- c(
  v_107="(v_107) The public should get to see all reviews once the article is published.",
  v_108="(v_108) The public should get to see the reviews and the article draft once the submission has been accepted or rejected.",
  v_109="(v_109) The public should get to see the article draft immediately, even before the reviews have been supplied (and see reworked versions later)."
)

#--- "how should reviewers be compensated":
monetary = df$v_55
certificate = df$v_58
reviewercompensation.items <- c("v_55", "v_58")
df[ ,factorize.name(reviewercompensation.items)] = data.frame(
    factor(df$v_55, levels = c(0, 1), labels = c("no", "yes")),
    factor(df$v_58, levels = c(0, 1), labels = c("no", "yes")))
reviewercompensation.df = df[ ,factorize.name(reviewercompensation.items)]
reviewercompensation.df.describe <- describe(reviewercompensation.df)

#--- age:
df$age = df$v_61
df$age[df$age < 16 | df$age > 100] = NA  # remove bad inputs
df$ageD = df$age / 10

#--- gender:
df$gender <- df$v_62
df$gender <- gsub("\\<M\\>|\\<m\\>|\\<male\\>|\\<Male\\>|\\<Make\\>",
                  "M", df$gender)
df$gender <- gsub("\\<F\\>|\\<f\\>|\\<female\\>|\\<Female\\>|\\<Woman\\>|\\<Femail\\>",
                  "F", df$gender)
df$gender[(!df$gender=="M" & !df$gender=="F") | is.na(df$gender)] <- "O"

#--- current position:
df$position = ifelse(df$v_63==1, "bscstud",
              ifelse(df$v_64==1, "mscstud",
              ifelse(df$v_65==1, "phdcand",
              ifelse(df$v_66==1, "postdoc",
              ifelse(df$v_67==1, "indust",
              ifelse(df$v_68==1, "nontenured",
              ifelse(df$v_69==1, "tenured",
              ifelse(df$v_70==1, "noresearch",  NA))))))))
df$tenured = (df$position == "tenured")
df$prof = (df$position %in% c("nontenured","tenured"))
df$positionf = factor(df$position,
                   levels = c( "bscstud",
                               "mscstud",
                               "phdcand",
                               "postdoc",
                               "indust",
                               "nontenured",
                               "tenured",
                               "noresearch"
                   ),
                   labels = c( "Bachelor\nstudent",
                               "Master\nstudent",
                               "PhD cand.\nResearcher",
                               "Postdoc",
                               "Industrial\nresearcher",
                               "Non-tenured\nprofessor",
                               "Tenured\nprofessor",
                               "No longger a\nscientific researcher"
                              )
                     )

#--- publishing and ICSE activity:
df$papers <-      ifelse(df$v_71 > 99, NA, df$v_71)  # avoid bad inputs
df$papers_icse <- ifelse(df$v_72 > 30, NA, df$v_72)  # avoid bad inputs
df$icse_pcs <-    ifelse(df$v_73 > 3,  NA, df$v_73)  # avoid bad inputs


#-------------------- plot functions:

do.plot.likert = function() {
    # for some reason, pdf() produced corrupted files. Saved them manually.
    title <- "One main purpose of peer review is to.."
    #pdf(file="img/purpose.pdf", width=18/2.54, height=11/2.54, pointsize=9)
    #png("img/purpose_.png", width=800, height=600, pointsize=9)
    plot(likert(purpose.likert)) + ggtitle(title)
    #dev.off()

    title <- "To which extent should the reviewing process be open?"
    #pdf("img/reviewopenness.pdf", width=18/2.54, height=6/2.54, pointsize=9)
    plot(likert(reviewopenness.likert)) + ggtitle(title)
    #dev.off()

    title <- "To which extent should the publication process be open?"
    #pdf("img/publicationopenness.pdf", width=19/2.54, height=6/2.54, pointsize=9)
    plot(likert(publicationopenness.likert)) + ggtitle(title)
    #dev.off()
}

do.plot.bar = function() {
    ggplot(reviewercompensation.df,
           aes(factor(reviewercompensation.df$v_55))) +
       geom_bar(aes(y = (..count..)/sum(..count..))) + 
       scale_y_continuous(labels=percent) +
       labs(title = "How reviewers should be treated",
            y = "Percent",
            x = "(v_55) Reviewers should receive a (quasi-)monetary compensation for their work\n (e.g. memberships, subscriptions, registration discounts, or money payment).")

     ggplot(reviewercompensation.df, 
            aes(factor(reviewercompensation.df$v_58))) +
        geom_bar(aes(y = (..count..)/sum(..count..))) + 
        scale_y_continuous(labels=percent) +
        labs(title = "How reviewers should be treated",
             y = "Percent", 
             x = "(v_58) Reviewers should receive showable proof for good reviewing work\n (e.g. public visibility of their review texts, or a reviewing quality certificate)")

     qplot(variable, data=position.df.m, geom="bar", fill=variable) + 
       labs(title = "", y = "Count", x = "Academic position.")
}

do.plot.xy = function() {
    cbbPalette <- c("#000000", "#E69F00", "#56B4E9", "#009E73", "#F0E442",  
                    "#777777", "#0072B2", "#D55E00", "#CC79A7", "#999999" )
    crossdata.age <- subset(df, !is.na(df$age))

    ggplot(crossdata.age, aes(v_104, age)) + 
      geom_jitter(aes(colour = v_104)) +
      scale_colour_manual(values=cbbPalette) +
      labs(title = "To which extent should the reviewing process be open?",
           y = "Age", x = "(v_104) Authors should know\nwho the reviewers of an article are (signed).")

    ggplot(crossdata.age, aes(v_107, age)) + 
      geom_jitter(aes(colour = v_107)) +
      scale_colour_manual(values=cbbPalette) +
      labs(title = "To which extent should the reviewing process be open?",
           y = "Age", x = "(v_107) The public should get to see\n all reviews once the article is published.")

    ggplot(crossdata.age, aes(v_109, age)) + 
      geom_jitter(aes(colour = v_109)) +
      scale_colour_manual(values=cbbPalette) +
      labs(title = "To which extent should the reviewing process be open?",
           y = "Age", x = "(v_109) The public should get to see the article draft immediately,\neven before the reviews have been supplied (and see reworked versions later).")

}

do.plot.box = function() {
    ggplot(crossdata.age,  aes(x=v_55,y=age,group=factor(v_55))) +
      geom_boxplot() +
      labs(title = "v_55 Reviewers should receive a (quasi-)monetary compensation for their work\n(e.g. memberships, subscriptions, registration discounts, or money payment). If so, which?",
           y = "Age", x = "Response") +
      theme(plot.title = element_text(size=11))

    ggplot(crossdata.age,  aes(x=v_58,y=age,group=factor(v_58))) +
      geom_boxplot() +
      labs(title = "v_58 Reviewers should receive showable proof for good reviewing work\n(e.g. public visibility of their review texts, or a reviewing quality certificate). If so, which?",
           y = "Age", x = "Response") +
      theme(plot.title = element_text(size=11))
}


do.plot.box.bwstrip = function() {
    myplot = bwplot(quality~percent, data=reviewstatusquo.df,
                    xlim=c(-3, 103), xlab="percent of all reviews",
                    panel=panel.bwstrip, whiskerpos=0)
    plotf(myplot, file="img/reviewquality", type="pdf",
          size=c(4, 3.5, 9, 96))
}

#-------------------- simple evaluations:

print.summaries = function() {
   cat("##### Distribution of current professional roles/positions:")
   print(table(df$position))

   cat("##### Gender distribution:")
   print(table(as.factor(df$gender)))

   cat("##### Age distribution (over decades, in percent):")
   tt = table(trunc(df$age/10))
   print(round(tt/sum(tt)*100))
   cat("n =", sum(tt), "\n")

   cat("##### Country distribution (in percent):")
   ct = sort(table(df$v_120), decreasing=T)
   print(round(ct/sum(ct)*100))
   cat("n =", sum(ct), "countriesN =", length(ct), "\n")

   cat("##### Reasons for faulty reviews (in percent of answers):")
   fr.answers = df$v_36
   frt = sort(table(fr.answers), decreasing=T)
   print(round(frt/sum(frt)*100))
   cat("answersN =", sum(!is.na(df$v_36)), "codesN =", sum(frt), "\n")


}

#-------------------- complex evaluations:

print.models5 = function(base.modelnum, dependent.var) {
   what = c("coefficients","adj.r.squared")
   types = c("A+P", "A:P", "A+T", "A:T", "A")
   N = length(types)
   rhs = c("ageD+prof", "ageD:prof", "ageD+tenured", "ageD:tenured", "ageD")
   cat("#########################\n")
   for (i in 1:N) {
      my.type = types[i]
      my.rhs = rhs[i]
      cat("##### linear model ", N*base.modelnum+i, " (", my.type, ") of ", 
          dependent.var, ":\n", sep="")
      formula = eval(parse(text=paste(dependent.var, "~", my.rhs)))
      print(summary(lm(formula, data=df))[what])
  }

}

print.models = function() {
   depvars = c("Pgood", "Punhelpful", "Pfaulty",
                "AknowR", "RknowA", "RknowR", "openness", "opennessChg",
                "pubreviews", "publicness",
                "monetary", "certificate")
   cat("########################################################\n")
   cat("########################################################\n")
   for (i in 1:length(depvars)) {
       print.models5(i-1, depvars[i])
   }
}

#-------------------- MAIN:

do.it.all = function() {
    do.plot.likert()
    do.plot.bar()
    do.plot.xy()
    do.plot.box()
    print.summaries()
    print.models()
}


cat("Packages and data loaded. Now call do.it.all().\n")
